/*
 * This program is copyright (c) 2007 Hortis-GRC SA.
 * 
 * This file is part of Sonar.
 * Sonar is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version.
 * 
 * Sonar is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with Sonar; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */
package ch.hortis.sonar.jpa;

import ch.hortis.sonar.model.JdbcData;
import org.hibernate.dialect.DB2Dialect;
import org.hibernate.dialect.HSQLDialect;
import org.hibernate.dialect.PostgreSQLDialect;
import org.hibernate.dialect.SQLServerDialect;

import javax.persistence.EntityManagerFactory;
import javax.persistence.PersistenceException;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

/**
 * Hibernate utils class to retreive an entitymanager factory for EJB3
 * persistance
 */
public class JPAUtil {

  private static Map<Integer, EntityManagerFactory> factories = new HashMap<Integer, EntityManagerFactory>();

  /**
   * Reset the created EntityManagerFactory objects in "cache"
   */
  protected static void reset() {
    for (EntityManagerFactory factory : factories.values()) {
      factory.close();
    }
    factories.clear();
  }

  protected static synchronized EntityManagerFactory getEntityManagerFactory( JdbcData jdbcData, Map<String, String> configuration ) {

    Integer key = jdbcData.hashCode();

    EntityManagerFactory factory = factories.get( key );
    if ( factory != null ) {
      return factory;
    }

    // other settings in /META-INF/persistence.xml
    Properties props = new Properties();
    if ( jdbcData.getDatasource() == null ) {
      props.put( "hibernate.connection.url", jdbcData.getUrl() );
      props.put( "hibernate.connection.driver_class", jdbcData.getDriverClassName() );
      props.put( "hibernate.connection.username", jdbcData.getUsername() );
      props.put( "hibernate.connection.password", jdbcData.getPassword() );
      props.put( "hibernate.dialect", getHibernateDialect( jdbcData.getUrl() ) );
    } else {
      props.put( "hibernate.connection.datasource", jdbcData.getDatasource() );
      props.put( "hibernate.dialect", getHibernateDialect( jdbcData.getDsDialect() ) );
    }
    String stats = System.getProperty( "sonar.db.statistics" );
    if ( "true".equalsIgnoreCase( stats ) ) {
      props.put( "hibernate.generate_statistics", "true" );
    } else {
      props.put( "hibernate.generate_statistics", "false" );
    }
    if ( jdbcData.isCacheEnabled() ) {
      props.put( "hibernate.cache.provider_class", "org.hibernate.cache.EhCacheProvider" );
      props.put( "hibernate.cache.use_second_level_cache", "true" );
      props.put( "hibernate.cache.use_query_cache", "false" );
    } else {
      props.put( "hibernate.cache.provider_class", "org.hibernate.cache.NoCacheProvider" );
      props.put( "hibernate.cache.use_second_level_cache", "false" );
      props.put( "hibernate.cache.use_query_cache", "false" );
    }
    props.put( "hibernate.connection.isolation", Integer.toString( jdbcData.getIsolationLevel() ) );

    if ( configuration != null ) {
      props.putAll( configuration );
    }
    factory = javax.persistence.Persistence.createEntityManagerFactory( "sonar", props );
    factories.put( key, factory );
    return factory;
  }

  protected static String getHibernateDialect( JdbcData.JDBCDialect dialect ) throws PersistenceException {
    if ( dialect == JdbcData.JDBCDialect.DB2 ) {
      return DB2Dialect.class.getName();
    } else if ( dialect == JdbcData.JDBCDialect.DERBY ) {
      return DerbyWithDecimalDialect.class.getName();
    } else if ( dialect == JdbcData.JDBCDialect.HSQLDB ) {
      return HSQLDialect.class.getName();
    } else if ( dialect == JdbcData.JDBCDialect.MSSQL ) {
      return SQLServerDialect.class.getName();
    } else if ( dialect == JdbcData.JDBCDialect.MYSQL ) {
      return MySqlWithDecimalDialect.class.getName();
    } else if ( dialect == JdbcData.JDBCDialect.ORACLE ) {
      return OracleWithDecimalDialect.class.getName();
    } else if ( dialect == JdbcData.JDBCDialect.POSTGRE ) {
      return PostgreSQLDialect.class.getName();
    }
    throw new PersistenceException( "Unsupported database for Dialect " + dialect );
  }

  /**
   * Hibernate dialect -> supported db mapping
   *
   * @param url the JDBC URL
   * @return a string containing the hibernate dialect class name to use
   * @throws PersistenceException if the given JDBC URL is not supported
   */
  protected static String getHibernateDialect( String url ) throws PersistenceException {
    if ( url.toLowerCase().startsWith( "jdbc:db2:" ) ) {
      return DB2Dialect.class.getName();
    } else if ( url.toLowerCase().startsWith( "jdbc:derby:" ) ) {
      return DerbyWithDecimalDialect.class.getName();
    } else if ( url.toLowerCase().startsWith( "jdbc:hsqldb:" ) ) {
      return HSQLDialect.class.getName();
    } else if ( url.toLowerCase().startsWith( "jdbc:microsoft:sqlserver:" ) ) {
      return SQLServerDialect.class.getName();
    } else if ( url.toLowerCase().startsWith( "jdbc:mysql:" ) ) {
      return MySqlWithDecimalDialect.class.getName();
    } else if ( url.toLowerCase().startsWith( "jdbc:oracle:" ) ) {
      return OracleWithDecimalDialect.class.getName();
    } else if ( url.toLowerCase().startsWith( "jdbc:postgresql:" ) ) {
      return PostgreSQLDialect.class.getName();
    }
    throw new PersistenceException( "Unsupported database for URL " + url );
  }
  
  /**
   * JDBC URL -> JDBC Dialect
   *
   * @param url the JDBC URL
   * @return a string containing the JDBC dialect identifier to use
   * @throws PersistenceException if the given JDBC URL is not supported
   */
  public static String getJDBCDialect( String url ) throws PersistenceException {
    if ( url.toLowerCase().startsWith( "jdbc:db2:" ) ) {
      return JdbcData.JDBCDialect.DB2.name();
    } else if ( url.toLowerCase().startsWith( "jdbc:derby:" ) ) {
      return JdbcData.JDBCDialect.DERBY.name();
    } else if ( url.toLowerCase().startsWith( "jdbc:hsqldb:" ) ) {
      return JdbcData.JDBCDialect.HSQLDB.name();
    } else if ( url.toLowerCase().startsWith( "jdbc:microsoft:sqlserver:" ) ) {
      return JdbcData.JDBCDialect.MSSQL.name();
    } else if ( url.toLowerCase().startsWith( "jdbc:mysql:" ) ) {
      return JdbcData.JDBCDialect.MYSQL.name();
    } else if ( url.toLowerCase().startsWith( "jdbc:oracle:" ) ) {
      return JdbcData.JDBCDialect.ORACLE.name();
    } else if ( url.toLowerCase().startsWith( "jdbc:postgresql:" ) ) {
      return JdbcData.JDBCDialect.POSTGRE.name();
    }
    throw new PersistenceException( "Unsupported database for URL " + url );
  }
}
