/*
 * This program is copyright (c) 2007 Hortis-GRC SA.
 *
 * This file is part of Sonar.
 * Sonar is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */
package ch.hortis.sonar.model;

import java.sql.Connection;

/**
 * Useful informations container for the JDBC connection.
 */
public class JdbcData {
  
  public final static String DEFAULT_SONAR_DS_JNDI_NAME = "java:comp/env/jdbc/sonar";
  public final static String DEFAULT_SONAR_DIALECT = JDBCDialect.DERBY.toString();
  
  private String url;
  private String driverClassName;
  private String username;
  private String password;
  private String datasource;
  private boolean cacheEnabled;
  private int isolationLevel;
  private JDBCDialect dsDialect;
  
  public enum JDBCDialect {
    DB2,DERBY,HSQLDB,MSSQL,MYSQL,ORACLE,POSTGRE;
    
    private static JDBCDialect fromString( String dialect ) throws IllegalArgumentException {
      for ( JDBCDialect dialectEnum :JDBCDialect.values() ) {
        if ( dialectEnum.toString().equalsIgnoreCase( dialect ) ) {
          return dialectEnum;
        }
      }
      throw new IllegalArgumentException( "Unable to map dialect " + dialect + " to one of the JDBCDialect enumeration value" );
    }
  }

  public JdbcData(String url, String driverClassName, String username, String password) {
    this.url = url;
    this.driverClassName = driverClassName;
    this.username = username;
    this.password = password;
    this.cacheEnabled = true;
    this.isolationLevel = Connection.TRANSACTION_READ_UNCOMMITTED;
  }

  public JdbcData( String datasource, String dsDialect ) {
    this.datasource = datasource;
    this.dsDialect = JDBCDialect.fromString( dsDialect );
    this.cacheEnabled = true;
    this.isolationLevel = Connection.TRANSACTION_READ_UNCOMMITTED;
  }
  
  public int getIsolationLevel() {
    return isolationLevel;
  }

  public void setIsolationLevel( int isolationLevel ) {
    this.isolationLevel = isolationLevel;
  }

  public boolean isCacheEnabled() {
    return this.cacheEnabled;
  }
  
  public void disableCache() {
    this.cacheEnabled = false;
  }

  public String getDatasource() {
    return datasource;
  }
  
  public JDBCDialect getDsDialect() {
    return dsDialect;
  }

  public String getUrl() {
    return url;
  }

  public String getDriverClassName() {
    return driverClassName;
  }

  public String getUsername() {
    return username;
  }

  public String getPassword() {
    if (password == null) {
      return "";
    }
    return password;
  }

  public int hashCode() {
    if ( getDatasource() != null ) return ( getDsDialect() + getDatasource() + isCacheEnabled() + getIsolationLevel() ).hashCode();
    return ( getUsername() + getPassword() + getUrl() + 
             getDriverClassName()  + isCacheEnabled() + getIsolationLevel() ).hashCode();
  }

  final public String toString() {
    final String separator = " # ";
    if ( getDatasource() != null ) return getDatasource() + separator + getDsDialect() + separator + isCacheEnabled() + separator + getIsolationLevel();
    return getUrl() + separator + getDriverClassName() + separator + getUsername() + 
                      separator + getPassword() + separator + isCacheEnabled() + separator + getIsolationLevel();
  }

  public static class Hsqldb extends JdbcData {
    public Hsqldb() {
      super("jdbc:hsqldb:mem:sonar", "org.hsqldb.jdbcDriver", "sa", null);
    }
  }
}