/*
 * This program is copyright (c) 2007 Hortis-GRC SA.
 * 
 * This file is part of Sonar.
 * Sonar is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version.
 * 
 * Sonar is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with Sonar; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */
package ch.hortis.sonar.model;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;
import org.hibernate.annotations.Immutable;

import javax.persistence.*;

@Immutable
@Cache(usage = CacheConcurrencyStrategy.READ_ONLY)
@Entity(name = "Metric")
@Table(name = "metrics")
@NamedQueries({@NamedQuery(name = Metric.SQL_SELECT_BY_NAME, query = "SELECT m FROM Metric m WHERE "
    + " m.name = :name"),
@NamedQuery(name = Metric.SQL_SELECT_ALL, query = "SELECT m FROM Metric m")
    })
public class Metric {

  public final static String SQL_SELECT_BY_NAME = "Metrics.selectByName";
  public static final String SQL_SELECT_ALL = "Metrics.selectAll";

  @Id
  @Column(name = "id")
  @SequenceGenerator(name = "METRICS_SEQ", sequenceName = "METRICS_SEQ")
  @GeneratedValue(strategy = GenerationType.AUTO, generator = "METRICS_SEQ")
  private Integer id;

  @Column(name = "name", updatable = false, nullable = false, length = 64)
  private String name;

  @Column(name = "description", updatable = false, nullable = true, length = 128)
  private String description;

  @Column(name = "value_type", updatable = false, nullable = false)
  @Enumerated(EnumType.ORDINAL)
  private MetricType type;

  @Column(name = "direction", updatable = false, nullable = true)
  private Integer direction;

  public Metric() {
  }

  public Metric(String name, MetricType type) {
    this.name = name;
    this.type = type;
  }

  public Metric(Metrics metric, MetricType type) {
    this(metric.getName(), type);
  }

  public Integer getId() {
    return id;
  }

  public void setId(Integer id) {
    this.id = id;
  }

  public String getName() {
    return name;
  }

  public void setName(String name) {
    this.name = name;
  }

  public MetricType getType() {
    return type;
  }

  public void setType(MetricType type) {
    this.type = type;
  }

  public static Number getParsedValue(MetricType type, Double value) {
    if (value == null) {
      return null;
    }
    switch (type) {
      case DOUBLE:
        return value;
      case FLOAT:
      case PERCENTAGE:
        return value.floatValue();
      case INT:
        return value.intValue();
      case LONG:
        return value.longValue();
      case SHORT:
        return value.shortValue();
      case BYTE:
        return value.byteValue();
      case DURATION_MS:
        return value.longValue();
    }
    return null;
  }

  public String getDescription() {
    return description;
  }

  public void setDescription(String description) {
    this.description = description;
  }

  public boolean isMetric(Metrics metric) {
    if (name == null && metric.getName() != null) {
      return false;
    } else if (!name.equals(metric.getName()))
      return false;

    return true;
  }


  public Integer getDirection() {
    return direction;
  }

  public void setDirection(Integer direction) {
    this.direction = direction;
  }

  public int hashCode() {
    // you pick a hard-coded, randomly chosen, non-zero, odd number
    // ideally different for each class
    return new HashCodeBuilder(17, 37).
        append(id).
        toHashCode();
  }

  public boolean equals(Object obj) {
    if (obj instanceof Metric == false) {
      return false;
    }
    if (this == obj) {
      return true;
    }
    Metric other = (Metric) obj;
    return new EqualsBuilder()
        .append(id, other.id)
        .isEquals();
  }

  public String toString() {
    return name;
  }

}
