/*
 * This program is copyright (c) 2007 Hortis-GRC SA.
 * 
 * This file is part of Sonar.
 * Sonar is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version.
 * 
 * Sonar is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with Sonar; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */
package ch.hortis.sonar.model;

import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;

import javax.persistence.*;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;

/**
 * Project snapshot group, allows to group a set of snapshots for a given maven plugin run
 */
@Entity
@Table(name = "snapshot_groups")
@NamedQueries({
@NamedQuery(name = SnapshotGroup.SQL_SELECT_LAST_PROCESSED, query = "SELECT s FROM SnapshotGroup s, MavenProject pr"
    + " WHERE s.mavenProject = pr.id" + " AND pr.id = :idProject AND s.last=true and s.processed=true"),
@NamedQuery(name = SnapshotGroup.SQL_SELECT_LAST_UNPROCESSED, query = "SELECT s FROM SnapshotGroup s, MavenProject pr"
    + " WHERE s.mavenProject = pr.id" + " AND pr.id = :idProject AND s.last=false and s.processed=false ORDER BY s.createdAt DESC"),
@NamedQuery(name = SnapshotGroup.SQL_SELECT_UNPROCESSED, query = "SELECT s FROM SnapshotGroup s"
    + " WHERE s.processed=false ORDER BY s.createdAt DESC"),
@NamedQuery(name = SnapshotGroup.SQL_SELECT_GROUPS_FOR_DATE, query = "SELECT s FROM SnapshotGroup s"
    + " WHERE s.createdAt < :createdAt AND s.processed = false"),
@NamedQuery(name = SnapshotGroup.SQL_SELECT_GROUPS_TO_PURGE, query = "SELECT s FROM SnapshotGroup s"
    + " WHERE s.purged=false AND s.last=false AND s.processed=true")
})
public class SnapshotGroup implements Serializable {

  private static final long serialVersionUID = 1292401756725899240L;

  public final static String SQL_SELECT_LAST_PROCESSED = "SnapshotGroup.selectLastProcessed";
  public final static String SQL_SELECT_LAST_UNPROCESSED = "SnapshotGroup.selectLast";
  public final static String SQL_SELECT_UNPROCESSED = "SnapshotGroup.selectUnprocessed";
  public final static String SQL_SELECT_GROUPS_FOR_DATE = "SnapshotGroup.selectGroupsForDate";
  public final static String SQL_SELECT_GROUPS_TO_PURGE = "SnapshotGroup.selectGroupsToPurge";

  @Id
  @Column(name = "id")
  @SequenceGenerator(name= "SNAPSHOT_GROUPS_SEQ", sequenceName = "SNAPSHOT_GROUPS_SEQ")
  @GeneratedValue(strategy = GenerationType.AUTO, generator = "SNAPSHOT_GROUPS_SEQ")
  private Integer id;

  @ManyToOne(fetch = FetchType.LAZY)
  @JoinColumn(name = "project_id", updatable = false, nullable = false)
  @Cache(usage = CacheConcurrencyStrategy.READ_ONLY)
  private MavenProject mavenProject;

  @Column(name = "islast")
  private Boolean last = Boolean.FALSE;

  @Column(name = "processed")
  private Boolean processed = Boolean.FALSE;
  
  @Column(name = "purged")
  private Boolean purged = Boolean.FALSE;

  @Temporal(TemporalType.TIMESTAMP)
  @Column(name = "created_at", updatable = false, nullable = false)
  private Date createdAt;

  @OneToMany(mappedBy = "snapshotGroup", fetch = FetchType.EAGER, cascade = {CascadeType.ALL})
  private Collection<Snapshot> snapshots = new ArrayList<Snapshot>();

  public Date getCreatedAt() {
    return createdAt;
  }

  public void setCreatedAt( Date createdAt) {
    this.createdAt = createdAt;
  }

  public Integer getId() {
    return id;
  }

  public void setId( Integer id ) {
    this.id = id;
  }

  public Boolean getLast() {
    return last;
  }

  public void setLast( Boolean last ) {
    this.last = last;
  }

  public MavenProject getMavenProject() {
    return mavenProject;
  }

  public void setMavenProject( MavenProject mavenProject ) {
    this.mavenProject = mavenProject;
  }

  public Boolean getProcessed() {
    return processed;
  }

  public void setProcessed( Boolean processed ) {
    this.processed = processed;
  }
  
  public Boolean getPurged() {
    return purged;
  }

  public void setPurged( Boolean purged ) {
    this.purged = purged;
  }

  public Collection<Snapshot> getSnapshots() {
    return snapshots;
  }

  public void setSnapshots( Collection<Snapshot> snapshots) {
    this.snapshots = snapshots;
  }

  public SnapshotGroup() {
  }

  public SnapshotGroup( MavenProject mavenProject, Boolean last, Boolean processed, Date createdAt ) {
    this.mavenProject = mavenProject;
    this.last = last;
    this.processed = processed;
    this.createdAt = createdAt;
  }

  public Snapshot getRootSnapshot() {
    Snapshot rootSnapshot = null;
    for (Snapshot snapshot : getSnapshots()) {
      if ( snapshot.getMavenProject().getId().equals( getMavenProject().getId() ) ) {
        rootSnapshot = snapshot;
      }
    }
    return rootSnapshot;
  }

  public List<Snapshot> getDirectChildren(Snapshot father) {
    List<Snapshot> children = new ArrayList<Snapshot>();
    for (Snapshot snapshot : getSnapshots()) {
      if ( snapshot.getMavenProject().getParent() != null &&
          snapshot.getMavenProject().getParent().getId().equals( father.getMavenProject().getId() ) ) {
        children.add(snapshot);
      }
    }
    return children;
  }
}
